package main

import (
	"fmt"
	"os"
	"strconv"
	"syscall"
	"time"
)

/*type amd64Regs struct {
	syscall.PtraceRegs;
	setter func (*syscall.PtraceRegs) os.Error;
}*/

func DumpRegs(regs *syscall.PtraceRegs) {
	fmt.Printf("R15:\t%d\n", regs.R15)
	fmt.Printf("R14:\t%d\n", regs.R14)
	fmt.Printf("R13:\t%d\n", regs.R13)
	fmt.Printf("R12:\t%d\n", regs.R12)
	fmt.Printf("R11:\t%d\n", regs.R11)
	fmt.Printf("R10:\t%d\n", regs.R10)
	fmt.Printf("R9:\t%d\n", regs.R9)
	fmt.Printf("R8:\t%d\n", regs.R8)
	fmt.Printf("RDI:\t%d\n", regs.Rdi)
	fmt.Printf("RSP:\t%d\n", regs.Rsp)
	fmt.Printf("RSI:\t%d\n", regs.Rsi)
	fmt.Printf("RBP:\t%d\n", regs.Rbp)
	fmt.Printf("RBX:\t%d\n", regs.Rbx)
	fmt.Printf("RCX:\t%d\n", regs.Rbx)
	fmt.Printf("RAX:\t%d\n", regs.Rax)
	fmt.Printf("RIP:\t%d\n", regs.Rip)

	fmt.Printf("RFLAGS:\t%d\n", regs.Rflags)
	fmt.Printf("TRAPNO:\t%d\n", regs.Trapno)
	fmt.Printf("ERR:\t%d\n", regs.Err)

	fmt.Printf("FS:\t%d\n", regs.Fs)
	fmt.Printf("GS:\t%d\n", regs.Gs)
	fmt.Printf("ES:\t%d\n", regs.Es)
	fmt.Printf("DS:\t%d\n", regs.Ds)
	fmt.Printf("CS:\t%d\n", regs.Cs)
	fmt.Printf("SS:\t%d\n", regs.Ss)
}

func main() {
	var pid int
	//var wpid int
	var regs syscall.PtraceRegs
	var status syscall.WaitStatus
	var rusage syscall.Rusage

	if len(os.Args) == 1 {
		fmt.Fprintf(os.Stderr, "usage: %s <pid>\n", os.Args[0])
		os.Exit(1)
	}
	pid, _ = strconv.Atoi(os.Args[1])

	// attach pid
	syscall.PtraceAttach(pid)
	fmt.Printf("[+] Attaching pid %d\n", pid)

	// singlestep pid
	syscall.PtraceSingleStep(pid)
	fmt.Printf("[+] SingleStep pid %d\n", pid)

	syscall.PtraceGetRegs(pid, &regs)
	fmt.Printf("[+] GetRegs pid %d\n", pid)
	DumpRegs(&regs)

	time.Sleep(10000 * time.Millisecond)

	wpid, err := syscall.Wait4(pid, &status, 0, &rusage)
	if err != nil {
		fmt.Fprintf(os.Stderr, "wait4(%d)\n", wpid)
		os.Exit(1)
	}

	// detach pid
	syscall.PtraceDetach(pid)
	fmt.Printf("[+] Detaching pid %d\n", pid)

	os.Exit(0)
}
